%% Create Table 5 
clear all
clc
addpath('..\Functions')

% Load data
load data\permno
load data\dates
load data\RET
load data\ME
load data\CUStock
load data\IK_Belo
load data\OVERHANG_AT
load data\BM
load data\HIRE
load data\TFP 
load data\OC_AT

% Loading the FF factor data to get the RFR;
load data\Factors_data\FF4F

start_date  = 196701;
end_date    = 201512;
dateRange   = Util_YYYYMMdates(start_date, end_date);
datesInd = (dates>=dateRange(1) & dates<=dateRange(end));

CUStock     = winsor(CUStock,0.01);
TFP         = winsor(TFP,0.01);
HIRE        = winsor(HIRE,0.01);
IK_Belo     = winsor(IK_Belo,0.01);
ME          = winsor(ME,0.01);
BM          = winsor(BM,0.01);
OVERHANG    = winsor(OVERHANG_AT,0.01);
OC_AT       = winsor(OC_AT,0.01);

dates       = dates(datesInd);
RET         = RET(datesInd,:);
ME          = ME(datesInd,:);
CUStock     = CUStock(datesInd,:);
TFP         = TFP(datesInd,:);
BM          = BM(datesInd,:);
IK_Belo     = IK_Belo(datesInd,:);
HIRE        = HIRE(datesInd,:);
OVERHANG    = OVERHANG(datesInd,:);
OC_AT       = OC_AT(datesInd,:);
RF          = FF4F(datesInd,6);
ERET        = RET - repmat(RF,[1 size(RET,2)]);

YYYY    = CRSPyear(dates);
MM      = CRSPmonth(dates);
YYYYMM  = YYYY*100+MM;

% Counting the number of time periods;
T   = size(ERET,1);
S   = size(ERET,2);

AnnERET = NaN(sum(MM==7)-1,S);
row_output=1;
for t=1:T
    if MM(t)==7
        for i=1:S
            cum_ret = 0;
                for tau=0:11
                    if t+tau<=T
                        cum_ret = cum_ret+log(1+ERET(t+tau,i));
                    end
                end
                AnnERET(row_output,i)=exp(cum_ret)-1;
        end
        row_output = row_output+1;
    end
end
AnnERET = real(AnnERET);
CUJune              = circshift(CUStock,3);
CUJune              = CUJune(MM==6,:);
TFPJune             = TFP(MM==6,:);
LagRetJune          = circshift(AnnERET,-1);
MEJune              = log(ME(MM==6,:));
BMJune              = BM(MM==6,:);
BMJune(BMJune<=0)   = NaN;
BMJune              = log(BMJune);
INVJune             = IK_Belo(MM==6,:);
HIREJune            = HIRE(MM==6,:);
OVERJune            = OVERHANG(MM==6,:);
OCJune              = OC_AT(MM==6,:);

stdCU    = nanstd(CUJune(:));
stdTFP   = nanstd(TFPJune(:));
stdRET   = nanstd(LagRetJune(:));
stdME    = nanstd(MEJune(:));
stdBM    = nanstd(BMJune(:));
stdINV   = nanstd(INVJune(:));
stdHIRE  = nanstd(HIREJune(:));
stdOVER  = nanstd(OVERJune(:));
stdOC    = nanstd(OCJune(:));

CUJune      = CUJune./stdCU;
TFPJune     = TFPJune./stdTFP;
LagRetJune  = LagRetJune./stdRET;
MEJune      = MEJune./stdME;
BMJune      = BMJune./stdBM;
INVJune     = INVJune./stdINV;
HIREJune    = HIREJune./stdHIRE;
OVERJune    = OVERJune./stdOVER;
OCJune      = OCJune./stdOC;

YYYYJune = YYYY(MM==6);

% Getting ready to run regressions;
Tr = size(AnnERET,1);
% Create column matricies to hold the output of each model;
betas1  = NaN(Tr-1,2);
betas2  = NaN(Tr-1,3);
betas3  = NaN(Tr-1,3);
betas4  = NaN(Tr-1,3);
betas5  = NaN(Tr-1,3);
betas6  = NaN(Tr-1,3);
betas7  = NaN(Tr-1,4);
betas8  = NaN(Tr-1,5);
betas9  = NaN(Tr-1,6);
betas10 = NaN(Tr-1,9);

for t=1:Tr-1
    tempERET    = AnnERET(t,:);    
    tempCU      = CUJune(t,:);    
    tempTFP     = TFPJune(t,:);
    tempLag     = LagRetJune(t,:);
    tempME      = MEJune(t,:);
    tempBM      = BMJune(t,:);
    tempINV     = INVJune(t,:);
    tempHIRE    = HIREJune(t,:);
    tempOVER    = OVERJune(t,:);    
    tempOC      = OCJune(t,:);
    
    tempData = [tempERET; tempCU; tempTFP; tempHIRE; tempINV; tempOVER; tempME; tempBM; tempLag];
    index = ~isnan(sum(tempData));
    tempData = tempData(:,index);

    for m=1:10
        if m == 1
            m1 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData(2:2,:)'],1); 
            betas1(t,1)=m1.beta(2);
            betas1(t,2)=m1.r2;
        elseif m==2
            m2 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 3],:)'],1);           
            betas2(t,1:2)=m2.beta(2:3);
            betas2(t,3)=m2.r2;
        elseif m==3
            m3 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 4],:)'],1);
            betas3(t,1:2)=m3.beta(2:3);
            betas3(t,3)=m3.r2;
        elseif m==4
            m4 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 5],:)'],1);
            betas4(t,1:2)=m4.beta(2:3);
            betas4(t,3)=m4.r2;
        elseif m==5
            m5 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 6],:)'],1);  
            betas5(t,1:2)=m5.beta(2:3);
            betas5(t,3)=m5.r2;
        elseif m==6
            m6 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 8],:)'],1);
            betas6(t,1:2)=m6.beta(2:3);
            betas6(t,3)=m6.r2;                
        elseif m==7
            m7 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 3 4],:)'],1);           
            betas7(t,1:3)=m7.beta(2:4);
            betas7(t,4)=m7.r2;    
        elseif m==8
            m8 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 3 4 5],:)'],1);
            betas8(t,1:4)=m8.beta(2:5);
            betas8(t,5)=m8.r2;  
        elseif m==9
            m9 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 3 4 5 6],:)'],1);
            betas9(t,1:5)=m9.beta(2:6);
            betas9(t,6)=m9.r2;    
        elseif m==10
            m10 = ols(tempData(1,:)', [ones(size(tempData(1,:)',1),1) tempData([2 3 4 5 6 7 8 9],:)'],1);
            betas10(t,1:8)=m10.beta(2:end);
            betas10(t,9)=m10.r2;
        end
    end
end

End_Year = 2014;
Start_Year = 1967;
betas1 = betas1(YYYYJune<=End_Year,:);
betas2 = betas2(YYYYJune<=End_Year,:);
betas3 = betas3(YYYYJune<=End_Year,:);
betas4 = betas4(YYYYJune<=End_Year,:);
betas5 = betas5(YYYYJune<=End_Year,:);
betas6 = betas6(YYYYJune<=End_Year,:);
betas7 = betas7(YYYYJune<=End_Year,:);
betas8 = betas8(YYYYJune<=End_Year,:);
betas9 = betas9(YYYYJune<=End_Year,:);
betas10 = betas10(YYYYJune<=End_Year,:);
TS_hac = 1;

ols_temp1_a = ols(betas1(:,1),ones(size(betas1,1),1),TS_hac);    
ols_temp2_a = ols(betas2(:,1),ones(size(betas2,1),1),TS_hac);    
ols_temp2_b = ols(betas2(:,2),ones(size(betas2,1),1),TS_hac);   
ols_temp3_a = ols(betas3(:,1),ones(size(betas3,1),1),TS_hac);    
ols_temp3_c = ols(betas3(:,2),ones(size(betas3,1),1),TS_hac);    
ols_temp4_a = ols(betas4(:,1),ones(size(betas4,1),1),TS_hac);    
ols_temp4_d = ols(betas4(:,2),ones(size(betas4,1),1),TS_hac);    
ols_temp5_a = ols(betas5(:,1),ones(size(betas5,1),1),TS_hac);   
ols_temp5_e = ols(betas5(:,2),ones(size(betas5,1),1),TS_hac);    
ols_temp6_a = ols(betas6(:,1),ones(size(betas6,1),1),TS_hac);    
ols_temp6_f = ols(betas6(:,2),ones(size(betas6,1),1),TS_hac);   
ols_temp7_a = ols(betas7(:,1),ones(size(betas7,1),1),TS_hac);    
ols_temp7_b = ols(betas7(:,2),ones(size(betas7,1),1),TS_hac);    
ols_temp7_c = ols(betas7(:,3),ones(size(betas7,1),1),TS_hac);    
ols_temp8_a = ols(betas8(:,1),ones(size(betas8,1),1),TS_hac);    
ols_temp8_b = ols(betas8(:,2),ones(size(betas8,1),1),TS_hac);    
ols_temp8_c = ols(betas8(:,3),ones(size(betas8,1),1),TS_hac);   
ols_temp8_d = ols(betas8(:,4),ones(size(betas8,1),1),TS_hac);   
ols_temp9_a = ols(betas9(:,1),ones(size(betas9,1),1),TS_hac);    
ols_temp9_b = ols(betas9(:,2),ones(size(betas9,1),1),TS_hac);    
ols_temp9_c = ols(betas9(:,3),ones(size(betas9,1),1),TS_hac);    
ols_temp9_d = ols(betas9(:,4),ones(size(betas9,1),1),TS_hac);    
ols_temp9_e = ols(betas9(:,5),ones(size(betas9,1),1),TS_hac);    
ols_temp10_a = ols(betas10(:,1),ones(size(betas10,1),1),TS_hac); 
ols_temp10_b = ols(betas10(:,2),ones(size(betas10,1),1),TS_hac); 
ols_temp10_c = ols(betas10(:,3),ones(size(betas10,1),1),TS_hac); 
ols_temp10_d = ols(betas10(:,4),ones(size(betas10,1),1),TS_hac); 
ols_temp10_e = ols(betas10(:,5),ones(size(betas10,1),1),TS_hac); 
ols_temp10_f = ols(betas10(:,6),ones(size(betas10,1),1),TS_hac); 
ols_temp10_g = ols(betas10(:,7),ones(size(betas10,1),1),TS_hac); 
ols_temp10_h = ols(betas10(:,8),ones(size(betas10,1),1),TS_hac); 
ols_temp10_i = ols(betas10(:,9),ones(size(betas10,1),1),TS_hac);

S = 100; % Report returns as a %;
tid = fopen('..\TablesAndFigures\Table_5.tex', 'w+');
fprintf(tid,'\\begin{tabular}{l c c c c c c c c c c} \\\\\n');
fprintf(tid,'\\toprule \n');
fprintf(tid,'\\hline \n');
fprintf(tid,' & (1) & (2) & (3) & (4) & (5) & (6) & (7) & (8) & (9) & (10) \\\\\n');
fprintf(tid,'\\midrule \n');
fprintf(tid,'CU & %2.2f & %2.2f & %2.2f & %2.2f & %2.2f & %2.2f & %2.2f & %2.2f & %2.2f & %2.2f \\\\\n',...
    ols_temp1_a.beta(1)*S,ols_temp2_a.beta(1)*S,ols_temp3_a.beta(1)*S,ols_temp4_a.beta(1)*S,ols_temp5_a.beta(1)*S,...
    ols_temp6_a.beta(1)*S,ols_temp7_a.beta(1)*S,ols_temp8_a.beta(1)*S,ols_temp9_a.beta(1)*S,ols_temp10_a.beta(1)*S);
fprintf(tid,' & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) \\\\\n',...
    ols_temp1_a.hac_tstat(1),ols_temp2_a.hac_tstat(1),ols_temp3_a.hac_tstat(1),ols_temp4_a.hac_tstat(1),ols_temp5_a.hac_tstat(1),...
    ols_temp6_a.hac_tstat(1),ols_temp7_a.hac_tstat(1),ols_temp8_a.hac_tstat(1),ols_temp9_a.hac_tstat(1),ols_temp10_a.hac_tstat(1));
fprintf(tid,'TFP &  & %2.2f &  &  &  & & %2.2f & %2.2f & %2.2f & %2.2f \\\\\n',...
    ols_temp2_b.beta(1)*S,ols_temp7_b.beta(1)*S,ols_temp8_b.beta(1)*S,ols_temp9_b.beta(1)*S,ols_temp10_b.beta(1)*S);
fprintf(tid,' &  &  (%2.2f) &  &  &  & & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) \\\\\n',...
    ols_temp2_b.hac_tstat(1),ols_temp7_b.hac_tstat(1),ols_temp8_b.hac_tstat(1),ols_temp9_b.hac_tstat(1),ols_temp10_b.hac_tstat(1));
fprintf(tid,'HIRE &  & & %2.2f &  &  & & %2.2f & %2.2f & %2.2f & %2.2f \\\\\n',...
    ols_temp3_c.beta(1)*S,ols_temp7_c.beta(1)*S,ols_temp8_c.beta(1)*S,ols_temp9_c.beta(1)*S,ols_temp10_c.beta(1)*S);
fprintf(tid,' &  &  & (%2.2f)  &  &  & & (%2.2f) & (%2.2f) & (%2.2f) & (%2.2f) \\\\\n',...
    ols_temp3_c.hac_tstat(1),ols_temp7_c.hac_tstat(1),ols_temp8_c.hac_tstat(1),ols_temp9_c.hac_tstat(1),ols_temp10_c.hac_tstat(1));
fprintf(tid,'I/K &  & &  & %2.2f &  & &  & %2.2f & %2.2f & %2.2f \\\\\n',...
    ols_temp4_d.beta(1)*S,ols_temp8_d.beta(1)*S,ols_temp9_d.beta(1)*S,ols_temp10_d.beta(1)*S);
fprintf(tid,' &  &  &  & (%2.2f)  &  & & & (%2.2f) & (%2.2f) & (%2.2f) \\\\\n',...
    ols_temp4_d.hac_tstat(1),ols_temp8_d.hac_tstat(1),ols_temp9_d.hac_tstat(1),ols_temp10_d.hac_tstat(1));
fprintf(tid,'OVER &  & &  &  & %2.2f  & & & & %2.2f & %2.2f \\\\\n',...
    ols_temp5_e.beta(1)*S,ols_temp9_e.beta(1)*S,ols_temp10_e.beta(1)*S);
fprintf(tid,' &  &  &  &  & (%2.2f)  & &  & & (%2.2f) & (%2.2f) \\\\\n',...
    ols_temp5_e.hac_tstat(1),ols_temp9_e.hac_tstat(1),ols_temp10_e.hac_tstat(1));
fprintf(tid,'$\\ln(\\text{B/M})$ &  & &  &  &  & %2.2f  & & & & %2.2f \\\\\n',...
    ols_temp6_f.beta(1)*S,ols_temp10_f.beta(1)*S);
fprintf(tid,' &  &  &  &  &   & (%2.2f) & & & & (%2.2f) \\\\\n',...
    ols_temp6_f.hac_tstat(1),ols_temp10_f.hac_tstat(1));
fprintf(tid,'$\\ln(\\text{ME})$ &  & &  &  &  & & & & & %2.2f \\\\\n',...
    ols_temp10_g.beta(1)*S);
fprintf(tid,' &  &  &  &  &   & & & & & (%2.2f) \\\\\n',...
    ols_temp10_g.hac_tstat(1));
fprintf(tid,'OC / AT &  & &  &  &  & & & & & %2.2f \\\\\n',...
    ols_temp10_h.beta(1)*S);
fprintf(tid,' &  &  &  &  &   & & & & & (%2.2f) \\\\\n',...
    ols_temp10_h.hac_tstat(1));
fprintf(tid,'$RET_{t-1}$ &  & &  &  &  & & & & & %2.2f \\\\\n',...
    ols_temp10_i.beta(1)*S);
fprintf(tid,' &  &  &  &  &   & & & & & (%2.2f) \\\\\n',...
    ols_temp10_i.hac_tstat(1));
fprintf(tid,'\\hline \n');
fprintf(tid, '$R^{2}$ & %0.3f & %0.3f & %0.3f & %0.3f & %0.3f & %0.3f & %0.3f & %0.3f & %0.3f & %0.3f \\\\\n',...
    mean(betas1(:,end)), mean(betas2(:,end)), mean(betas3(:,end)), mean(betas4(:,end)),...
    mean(betas5(:,end)),mean(betas6(:,end)),mean(betas7(:,end)),mean(betas8(:,end)),mean(betas9(:,end)),...
    mean(betas10(:,end))); 
fprintf(tid,'\\bottomrule \n');
fprintf(tid,'\\end{tabular} \n');
