%% The purpose of this to produce Tables 1, 2, 3, and 4 in the paper;

clc
clear all
close all

addpath('..\Functions')

NIND = 57;                               
ret_span = [1967+1/12:1/12:2015+12/12]';  
excluded_assets=[1:5 47:50 55:57]';
breakpoint_percentiles=[10;90];
start_date_sample = 1967+7/12;
spec_span     = [start_date_sample:(1/12):ret_span(end)]';
% Read capacity utilization data
[years, months, span, N, T_all, caputils_lvl, industry_names]=data_capu(NIND, excluded_assets);   
[factors_ff3f]=data_factor(ret_span, start_date_sample);

sort_measures = caputils_lvl; 

[rets, retxs, mvs, lmvs, nfirms, ind_data] = data_indret(ret_span, N, NIND, excluded_assets);
[univ_N_PORTFOLIO, univ_portfolioSorts]=port_univ_sort(breakpoint_percentiles,...
months, N, sort_measures, span, ret_span, start_date_sample);
[univ_portfolioAssignments]=port_assign(univ_portfolioSorts);

[univ_portfolioRetsVW]=port_retsCU(1, lmvs,...
univ_N_PORTFOLIO, nfirms, univ_portfolioAssignments, rets, retxs, ret_span, start_date_sample);
[univ_portfolioRetsEW]=port_retsCU(0, lmvs,...
univ_N_PORTFOLIO, nfirms, univ_portfolioAssignments, rets, retxs, ret_span, start_date_sample);
    
%% Table 4;

count_ind=zeros(1,size(setdiff(1:NIND,excluded_assets),2));
min_ind=zeros(1,size(setdiff(1:NIND,excluded_assets),2));
max_ind=zeros(1,size(setdiff(1:NIND,excluded_assets),2));

for j=1:size(univ_portfolioAssignments,2)
    for i=1:size(univ_portfolioAssignments,1)
        if isnan(univ_portfolioAssignments(i,j))==0
            count_ind(j)=count_ind(j)+1;
        end
        if isnan(univ_portfolioAssignments(i,j))==0 && univ_portfolioAssignments(i,j)==1
            min_ind(j)=min_ind(j)+1;
        end
        if isnan(univ_portfolioAssignments(i,j))==0 && univ_portfolioAssignments(i,j)==size(breakpoint_percentiles,1)+1
            max_ind(j)=max_ind(j)+1;
        end
    end
end
table_data = zeros(size(univ_portfolioAssignments,2),3);
table_data = [count_ind' (min_ind./count_ind)'*100 (max_ind./count_ind)'*100];

ind_names = cell(size(industry_names,1),1);
for i=1:size(industry_names,1)
    ind_names{i}=strrep(industry_names{i}, '; s.a. CAPUTL', '');
    ind_names{i}=strrep(ind_names{i}, ' (NAICS = ', '');
    ind_names{i}=strrep(ind_names{i}, ')', '');
    ind_names{i}=strrep(ind_names{i}, '0', '');
    ind_names{i}=strrep(ind_names{i}, '1', '');
    ind_names{i}=strrep(ind_names{i}, '2', '');
    ind_names{i}=strrep(ind_names{i}, '3', '');
    ind_names{i}=strrep(ind_names{i}, '4', '');
    ind_names{i}=strrep(ind_names{i}, '5', '');
    ind_names{i}=strrep(ind_names{i}, '6', '');
    ind_names{i}=strrep(ind_names{i}, '7', '');
    ind_names{i}=strrep(ind_names{i}, '8', '');
    ind_names{i}=strrep(ind_names{i}, '9', '');
    ind_names{i}=strrep(ind_names{i}, '-', '');
    
    ind_names{i}=strtrim(ind_names{i});
    if i==3
        ind_names{i}=strcat(ind_names{i},')');
    end
    temp=cell2mat(ind_names{i});
    if strcmp(temp(end-1:end),' ,')
        temp=temp(1:end-2);
        ind_names{i}=cellstr(temp);
    elseif strcmp(temp(end-2:end),' ,,')
        temp=temp(1:end-3);
        ind_names{i}=cellstr(temp);
    end
end
RowLabels = cell(size(ind_names,1),1);
for i = 1:size(ind_names,1)
    RowLabels{i} = cell2mat([ind_names{i}]);
end
ColLabels={'n', 'Min', 'Max'};      

N=5; 
[sortedMin,sortingIndicesMin] = sort(table_data(:,2),'descend');
MinValues = sortedMin(1:N);
MinValueIndices = sortingIndicesMin(1:N);

[sortedMax,sortingIndicesMax] = sort(table_data(:,3),'descend');
MaxValues = sortedMax(1:N);
MaxValueIndices = sortingIndicesMax(1:N);

min_table = zeros(N, 2);
min_names = cell(N, 1);
max_table = zeros(N, 2);
max_names = cell(N, 1);

for i=1:N
    for j=1:size(ind_names,1)
        if j==MinValueIndices(i,1)
            min_table(i,1)=table_data(j,2);
            min_table(i,2)=table_data(j,3);
            min_names{i,1} = cell2mat([ind_names{j}]);
        end
    end
end

for i=1:N
    for j=1:size(ind_names,1)
        if j==MaxValueIndices(i,1)
            max_table(i,1)=table_data(j,3);
            max_table(i,2)=table_data(j,2);
            max_names{i,1} = cell2mat([ind_names{j}]);
        end
    end
end

% Part 5 - Constructing the table;
slb=char(91);
srb=char(93);
rlb=char(40);
rrb=char(41);
clb=char(123);
crb=char(125);
per=char(37);
ast=char(42);
star=char(35);

tid = fopen('..\TablesAndFigures\Table_4.tex', 'w+');
fprintf(tid, '\\small \n');
fprintf(tid, '\\begin{tabularx}{\\textwidth}{@{\\extracolsep{\\fill}} lll c lll } \n');
fprintf(tid, '\\toprule ');
fprintf(tid, '\\hline \n');
fprintf(tid, '\\text{Industry} &\\text{Sector} & \\text{Freq.}& &\\text{Industry} &\\text{Sector} &  \\text{Freq.} \\\\\n');
fprintf(tid, '\\midrule \n');
fprintf(tid, '\\multicolumn{3}{c}{\\text{Panel A: Low capacity utilization portfolio}} & &\\multicolumn{3}{c}{\\text{Panel B: High capacity utilization portfolio}} \\\\\n');
fprintf(tid, '\\cline{1-3} \\cline{5-7}\n');
for i=1:N
fprintf(tid, '%s & %s & %2.2f & & %s & %s & %2.2f \\\\\n',min_names{i}, util_sector(cell2mat(min_names(i))), min_table(i,1),...
    max_names{i}, util_sector(cell2mat(max_names(i))), max_table(i,1));
end
fprintf(tid, '\\bottomrule \n');
fprintf(tid, '\\end{tabularx}\n');
fclose(tid);

%% Creating Table

mean_VW = mean(univ_portfolioRetsVW)*100;
std_VW  = std(univ_portfolioRetsVW)*100;
mean_EW = mean(univ_portfolioRetsEW)*100;
std_EW  = std(univ_portfolioRetsEW)*100;

mean_spread_VW  = mean(univ_portfolioRetsVW(:,1)-univ_portfolioRetsVW(:,end))*100;
std_spread_VW   = std(univ_portfolioRetsVW(:,1)-univ_portfolioRetsVW(:,end))*100;
mean_spread_EW  = mean(univ_portfolioRetsEW(:,1)-univ_portfolioRetsEW(:,end))*100;
std_spread_EW   = std(univ_portfolioRetsEW(:,1)-univ_portfolioRetsEW(:,end))*100;

o_spread_VW = ols(univ_portfolioRetsVW(:,1)-univ_portfolioRetsVW(:,end),ones(size(univ_portfolioRetsVW(:,1),1),1),3);
o_spread_EW = ols(univ_portfolioRetsEW(:,1)-univ_portfolioRetsEW(:,end),ones(size(univ_portfolioRetsEW(:,1),1),1),3);
 
%Create the results file'
tid = fopen('..\TablesAndFigures\Table_1.tex', 'w+');
fprintf(tid, '\\begin{tabularx}{\\textwidth}{@{\\extracolsep{\\fill}} lcccccc } \n');
fprintf(tid, '\\toprule \n');
fprintf(tid, '\\hline \n');
fprintf(tid, ' & & \\multicolumn{2}{c}{\\text{Value-weighted}} & & \\multicolumn{2}{c}{\\text{Equal-weighted}} \\\\\n');
fprintf(tid, '\\cline{3-4}'); fprintf(tid, '\\cline{6-7 }');  
fprintf(tid, 'Portfolio & & Mean & SD & & Mean & SD \\\\\n');
fprintf(tid, '\\midrule \n');
fprintf(tid, 'Low (L) & & %2.2f & %2.2f & & %2.2f & %2.2f \\\\\n',mean_VW(1)*12,std_VW(1)*sqrt(12) ,mean_EW(1)*12,std_EW(1)*sqrt(12));
fprintf(tid, 'Medium & & %2.2f & %2.2f & & %2.2f & %2.2f \\\\\n',mean_VW(2)*12,std_VW(2)*sqrt(12) ,mean_EW(2)*12,std_EW(2)*sqrt(12));
fprintf(tid, 'High (H) & & %2.2f & %2.2f & & %2.2f & %2.2f \\\\\n',mean_VW(3)*12,std_VW(3)*sqrt(12) ,mean_EW(3)*12,std_EW(3)*sqrt(12));
fprintf(tid, '\\\\\n');
fprintf(tid, 'Spread & & %2.2f & %2.2f & & %2.2f & %2.2f \\\\\n',mean_spread_VW*12, std_spread_VW*sqrt(12) ,mean_spread_EW*12,std_spread_EW*sqrt(12));
fprintf(tid, '(L-H)  & & (%2.2f) &  & & (%2.2f) & \\\\\n',o_spread_VW.hac_tstat, o_spread_EW.hac_tstat);
fprintf(tid, '\\bottomrule \n');
fprintf(tid, '\\end{tabularx} \n');
fclose(tid);

%% Creating Table 2

qtrReturnsVW = NaN(size(univ_portfolioRetsVW,1)/3,size(univ_portfolioRetsVW,2)+1);
qtrMKTRFVW  = NaN(size(univ_portfolioRetsVW,1)/3,1);
qtrRF  = NaN(size(univ_portfolioRetsVW,1)/3,1);
for j=1:size(univ_portfolioRetsVW,2)
    count = 1;
    for i=1:3:size(univ_portfolioRetsVW,1)
        temp_sumVW = log(1+univ_portfolioRetsVW(i,j))+log(1+univ_portfolioRetsVW(i+1,j))+log(1+univ_portfolioRetsVW(i+2,j));
        temp_sumMKT= log(1+factors_ff3f(i,2)) + log(1+factors_ff3f(i+1,2)) + log(1+factors_ff3f(i+2,2));
        temp_sumRF = log(1+factors_ff3f(i,end)) + log(1+factors_ff3f(i+1,end)) + log(1+factors_ff3f(i+2,end));
        qtrReturnsVW(count,j)=(exp(temp_sumVW)-1);
        qtrMKTRFVW(count,1) = (exp(temp_sumMKT)-1);
        qtrRF(count,1) = (exp(temp_sumRF)-1);
        count = count+1;
    end
end
qtrReturnsVW(:,4)=qtrReturnsVW(:,1)-qtrReturnsVW(:,3);

load Data\quarterly_tfp.csv
% CAPM;
TFP_M_1_L=ols(qtrReturnsVW(:,1)-qtrRF,[ones(size(qtrReturnsVW,1),1) qtrMKTRFVW],3);
TFP_M_1_M=ols(qtrReturnsVW(:,2)-qtrRF,[ones(size(qtrReturnsVW,1),1) qtrMKTRFVW],3);
TFP_M_1_H=ols(qtrReturnsVW(:,3)-qtrRF,[ones(size(qtrReturnsVW,1),1) qtrMKTRFVW],3);
TFP_M_1=ols(qtrReturnsVW(:,4),[ones(size(qtrReturnsVW,1),1) qtrMKTRFVW],3);
TFP_col = 3; 
TFP_C_1_L=ols(qtrReturnsVW(:,1)-qtrRF,[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,TFP_col)/100],3);
TFP_C_1_M=ols(qtrReturnsVW(:,2)-qtrRF,[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,TFP_col)/100],3);
TFP_C_1_H=ols(qtrReturnsVW(:,3)-qtrRF,[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,TFP_col)/100],3);
TFP_C_1=ols(qtrReturnsVW(:,4),[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,TFP_col)/100],3);
col_index_2 = 11;
TFP_L_1=ols(qtrReturnsVW(:,4),[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,col_index_2)/100],3);
TFP_L_1_L=ols(qtrReturnsVW(:,1)-qtrRF,[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,col_index_2)/100],3);
TFP_L_1_M=ols(qtrReturnsVW(:,2)-qtrRF,[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,col_index_2)/100],3);
TFP_L_1_H=ols(qtrReturnsVW(:,3)-qtrRF,[ones(size(qtrReturnsVW,1),1) quarterly_tfp(:,col_index_2)/100],3);

%Create the results file;
tid = fopen('..\TablesAndFigures\Table_2.tex', 'w+');

fprintf(tid, '\\begin{tabularx}{\\textwidth}{@{\\extracolsep{\\fill}} ll cc c cc c cc } \n');
fprintf(tid, '\\toprule \n ');
fprintf(tid, '\\hline \n');
fprintf(tid, ' & & \\multicolumn{2}{c}{\\text{Market returns}} & & \\multicolumn{2}{c}{\\text{Util.-adjusted TFP}} & & \\multicolumn{2}{c}{\\text{Labor productivity}} \\\\\n');
fprintf(tid, '\\cline{3-4} \\cline{6-7} \\cline{9-10} \n');
fprintf(tid, 'Portfolio & & $\\beta$ & $t(\\beta)$ & & $\\beta$ & $t(\\beta)$ & & $\\beta$ & $t(\\beta)$ \\\\\n');
fprintf(tid, '\\midrule \n');
fprintf(tid, 'Low (L) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) \\\\\n',TFP_M_1_L.beta(2),TFP_M_1_L.hac_tstat(2),...
    TFP_C_1_L.beta(2),TFP_C_1_L.hac_tstat(2),...
    TFP_L_1_L.beta(2),TFP_L_1_L.hac_tstat(2));
fprintf(tid, 'Medium & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) \\\\\n',TFP_M_1_M.beta(2),TFP_M_1_M.hac_tstat(2),...
    TFP_C_1_M.beta(2),TFP_C_1_M.hac_tstat(2),...
    TFP_L_1_M.beta(2),TFP_L_1_M.hac_tstat(2));
fprintf(tid, 'High (H) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) \\\\\n',TFP_M_1_H.beta(2),TFP_M_1_H.hac_tstat(2),...
    TFP_C_1_H.beta(2),TFP_C_1_H.hac_tstat(2),...
    TFP_L_1_H.beta(2),TFP_L_1_H.hac_tstat(2));
fprintf(tid, '\\\\\n');
fprintf(tid, 'Spread (L-H)& & %2.2f & (%2.2f) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) \\\\\n',TFP_M_1.beta(2),TFP_M_1.hac_tstat(2),...
    TFP_C_1.beta(2),TFP_C_1.hac_tstat(2),...
    TFP_L_1.beta(2),TFP_L_1.hac_tstat(2));
fprintf(tid, 'Intercept & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) & & %2.2f & (%2.2f) \\\\\n',TFP_M_1.beta(1)*100*4,TFP_M_1.hac_tstat(1),...
    TFP_C_1.beta(1)*100*4,TFP_C_1.hac_tstat(1),...
    TFP_L_1.beta(1)*100*4,TFP_L_1.hac_tstat(1));
fprintf(tid, '\\bottomrule \n');
fprintf(tid, '\\end{tabularx} \n');
fclose(tid);

%% Creating Table 3
[chars_portfolioChars, chars_LabelsCol, chars_LabelsRow, chars_parentheses]=data_chars(caputils_lvl,...
span, ret_span, NIND, excluded_assets, univ_portfolioAssignments,...
univ_N_PORTFOLIO, rets, lmvs, nfirms, start_date_sample);
CharsSubset = NaN(17,5);
CharLabelSubset = cell(17,1);
for i=1:size(CharsSubset,1)
    if i==1
        CharsSubset(i,:) = chars_portfolioChars(1,:);
        CharLabelSubset{i}='CU (\%)';
    elseif i==2 
        CharsSubset(i,:) = chars_portfolioChars(2,:);
        CharLabelSubset{i}='$N \ (Stocks)$';
    elseif i==3
        CharsSubset(i,:) = chars_portfolioChars(3,:);
        CharLabelSubset{i}='$N \ (Ind.)$';
    elseif i==4
        CharsSubset(i,:) = [chars_portfolioChars(19,1:end-1)./1000 chars_portfolioChars(19,end)];
        CharLabelSubset{i}='ME (\$b)';
    elseif i==5 
        CharsSubset(i,:) = chars_portfolioChars(5,:);
        CharLabelSubset{i}='BE / ME';
    elseif i==6 
        CharsSubset(i,:) = chars_portfolioChars(16,:);
        CharLabelSubset{i}='ROA';
    elseif i==7 
        CharsSubset(i,:) = chars_portfolioChars(8,:);
        CharLabelSubset{i}='GP / Assets';
    elseif i==8 
        CharsSubset(i,:) = chars_portfolioChars(4,:);
        CharLabelSubset{i}='Asset Growth (\%)';
    elseif i==9 
        CharsSubset(i,:) = chars_portfolioChars(10,:);
        CharsSubset(i,1:end-1) = CharsSubset(i,1:end-1);
        CharLabelSubset{i}='Inventory growth (\%)';
    elseif i==10 
        CharsSubset(i,:) = chars_portfolioChars(12,:);
        CharLabelSubset{i}='I / K';
    elseif i==11
        CharsSubset(i,:) = chars_portfolioChars(18,:);
        CharLabelSubset{i}='IVOL (\%)';
    elseif i==12 
        CharsSubset(i,:) = chars_portfolioChars(17,:);
        CharLabelSubset{i}='TFP';
    elseif i==13 
        CharsSubset(i,:) = chars_portfolioChars(9,:);
        CharLabelSubset{i}='Hire Rate (\%)';
    elseif i==14 
        CharsSubset(i,:) = chars_portfolioChars(11,:);
        CharLabelSubset{i}='R\&D / ME';
    elseif i==15 
        CharsSubset(i,:) = chars_portfolioChars(15,:);
        CharLabelSubset{i}='OC / AT';
    elseif i==16 
        CharsSubset(i,:) = chars_portfolioChars(13,:);
        CharLabelSubset{i}='Leverage';
    elseif i==17 
        CharsSubset(i,:) = chars_portfolioChars(6,:);
        CharLabelSubset{i}='Debt growth (\%)';
    elseif i==18 
        CharsSubset(i,:) = chars_portfolioChars(7,:);
        CharsSubset(i,1:end-1) = CharsSubset(i,1:end-1)*100;
        CharLabelSubset{i}='Equity issue. (\%)';
    end
end
tid = fopen('..\TablesAndFigures\Table_3.tex', 'w+');
fprintf(tid, '\\begin{tabularx}{\\textwidth}{@{\\extracolsep{\\fill}} lcccccccccc } \n');
fprintf(tid, '\\toprule \n');
fprintf(tid, '\\hline \n');
fprintf(tid, ' & & \\text{Low (L)} & & \\text{Medium} & & \\text{High (H)} & & Diff(L-H) & & \\textit{t}(Diff) \\\\\n');
fprintf(tid, '\\midrule \n');
    fprintf(tid, '\\multicolumn{11}{c}{\\text{Panel A: Portfolio constituents}} \\\\\n');
fprintf(tid, '\\midrule \n');     
for i=1:size(CharsSubset,1)
    % Round assets and N different
    if i==1 || i==2 || i==3 % No spread for N's
        fprintf(tid, '%s & & %2.0f & & %2.0f & & %2.0f & & & & \\\\\n',CharLabelSubset{i},CharsSubset(i,1),CharsSubset(i,2),CharsSubset(i,3));
    elseif i==4
        fprintf(tid, '\\midrule \n');
            fprintf(tid, '\\multicolumn{11}{c}{\\text{Panel B: Portfolio characteristics}} \\\\\n');
        fprintf(tid, '\\midrule \n');
        fprintf(tid, '%s & & %2.2f & & %2.2f & & %2.2f & & %2.2f & & (%2.2f) \\\\\n',CharLabelSubset{i},CharsSubset(i,1)/1000,CharsSubset(i,2)/1000,CharsSubset(i,3)/1000,CharsSubset(i,4)/1000,CharsSubset(i,5));
    else
        fprintf(tid, '%s & & %2.2f & & %2.2f & & %2.2f & & %2.2f & & (%2.2f) \\\\\n',CharLabelSubset{i},CharsSubset(i,1),CharsSubset(i,2),CharsSubset(i,3),CharsSubset(i,4),CharsSubset(i,5)); 
    end
end
fprintf(tid, '\\bottomrule \n');
fprintf(tid, '\\end{tabularx} \n');
fclose(tid); 