Libname CRSP "<LOCAL-DIRECTORY>\Data";

* Preparing the CRSP data;
%Let KeepVars = PERMNO PERMCO DATE SHRCD EXCHCD PRC SHROUT RET RETX SICCD NAICS;

Proc Sort Data=CRSP.CRSP_MONTHLY (Keep=&KeepVars) Out=CRSP NoDup;
	By _All_;
Run;
Data CRSP;
	Set CRSP;
		If SHROUT > 0; 
		If Missing(RET) Then RET=.;
		If Missing(RETX) Then RETX=.;
		If Missing(RET) OR Missing(RETX) Or Missing(PERMNO) Then Delete;
		If SHRCD in (10,11) AND EXCHCD in (1,2,3);
		If RET = -66 OR RET = -77 OR RET = -88 OR RET = -99 Then Delete;	
Run;
Data CRSP;
	Set CRSP;
		PRC=ABS(PRC);
		mktVal = PRC*SHROUT;
		Year = Year(Date);
		Month = Month(Date);
		YearMonth = Year*100+Month;
		ME = PRC*SHROUT;
Run;
Data CRSP1;
	Set CRSP;
		NAICS2 = substr(NAICS,1,2)*1;
		NAICS3 = substr(NAICS,1,3)*1;
		NAICS4 = substr(NAICS,1,4)*1; 
		NAICS5 = substr(NAICS,1,5)*1; 
		NAICS6 = substr(NAICS,1,6)*1; 
		SIC4 = SICCD;
		SIC3 = int(SICCD/10);
		IF mod(sic4,10) = 0 Then is_SIC3=1;
			Else is_SIC3=0;
Run;
Proc Sort Data=CRSP1 OUT=CRSP2;
	By Permno YearMonth;
Run;
PROC SQL;
	Create Table Last_Obs As
	Select * 
	From CRSP2
	Group By Permno
	Having YearMonth=Max(YearMonth);
Quit;
Proc SQL;
	Create Table CRSP3 AS
	Select a.Permno, a.Date, a.YearMonth, a.Year, a.Month, a.SIC4, a.SIC3, a.is_SIC3, a.NAICS, a.RET, a.RETX, a.MKTVAL, 
		a.NAICS2 as NAICS2_Sample, b.NAICS2 as NAICS2_Last,
		a.NAICS3 as NAICS3_Sample, b.NAICS3 as NAICS3_Last,
		a.NAICS4 as NAICS4_Sample, b.NAICS4 as NAICS4_Last,
		a.NAICS5 as NAICS5_Sample, b.NAICS5 as NAICS5_Last,
		a.NAICS6 as NAICS6_Sample, b.NAICS6 as NAICS6_Last
	From CRSP2 as a Left Join Last_Obs as b
	On a.Permno=b.Permno
	Order By a.Permno, a.YearMonth;
Quit;
Data CRSP3 (Keep=Permno Date YearMonth Year Month MKTVAL SIC4 SIC3 is_SIC3 NewN2 NewN3 NewN4 NewN5 NewN6 Ret RETX
		rename=(NewN2=NAICS2 NewN3=NAICS3 NewN4=NAICS4 NewN5=NAICS5 NewN6=NAICS6));
	Set CRSP3;
		If Missing(NAICS2_Sample)=1 And NAICS2_Sample ~= 0 Then NewN2=NAICS2_Last;
			Else NewN2=NAICS2_Sample;
		If Missing(NAICS3_Sample)=1 And NAICS3_Sample ~= 0 Then NewN3=NAICS3_Last;
			Else NewN3=NAICS3_Sample;
		If Missing(NAICS4_Sample)=1 And NAICS4_Sample ~= 0 Then NewN4=NAICS4_Last;
			Else NewN4=NAICS4_Sample;
		If Missing(NAICS5_Sample)=1 And NAICS5_Sample ~= 0 Then NewN5=NAICS5_Last;
			Else NewN5=NAICS5_Sample;
		If Missing(NAICS6_Sample)=1 And NAICS6_Sample ~= 0 Then NewN6=NAICS6_Last;
			Else NewN6=NAICS6_Sample;
Run;
* Assigning each firm to the appropriate NAICS industry (using the NAICS-to-SIC crosswalk when NAICS is unavailable);
%INCLUDE "..\Functions\naics_to_sic.sas";

*Merging in cashflow characteristics from the pre-computed files (see the readme file on what is required!);
Proc SQL;
	Create Table CRSP4 As
	Select a.*, b.AT_Growth, b.BEME, b.Debt_Growth,  b.EI_BE, b.GP_AT, b.Hire_Rate As Hire_Rate,
				b.XRD_ME As XRD_ME, b.INVT_Growth,  b.INV_AT, b.Debt_AT as Debt_AT, b.IK as IK, b.OC_AT,
				b.ROA, b.TFP
	From CRSP4 as a Left Join CRSP.Characteristics as b
	On a.Permno=b.permno And b.Start_Date<=a.Date<=b.End_Date
	Order By a.Permno, a.YearMonth;
Quit;

* Merge in IVOl and ME data;
Proc SQL;
	Create Table CRSP4 As
	Select a.*, b.AHXZ_Ivol_M As Ivol, b.ME As ME
	From CRSP4 as a Left Join CRSP.IVOL_Monthly as b
	On a.Permno=b.permno And Month(a.Date)=Month(b.Date) And Year(a.Date)=Year(b.Date)
	Order By a.Permno, a.YearMonth;
Quit;

*Making sure there are no duplicates before calculating averages;
Proc Sort Data=CRSP4 NoDupKey;
	By Permno Year Month;
Run;

*Creating the CSV files for each industry;
%Macro IndLoop;
	%Do i=5 %To 46;
			Data CRSP5;
				Set CRSP4;
					If Industry%eval(&i)=1;
						Proc Sort;
							By Permno YearMonth;
			Run;
			
			Data CRSP5;
				Set CRSP5;
					Lag_Permno=Lag(Permno);
					Lag_Date=Lag(Date);
					Lag_MktVal=Lag(MKTVAL);
					Format Lag_Date YYMMDDN8.;
			Run;
			
			Data CRSP5 (Drop=Lag_Permno Lag_Date);
				Set CRSP5;
					If intck('day',Lag_Date,Date)<=40 & Permno=Lag_Permno Then Output;
					PRoc Sort;
						By YearMonth;
			Run;

			Proc SQL;
				Create Table MktVal As
				Select YearMonth, Sum(Lag_MktVal) As Total_LagMktVal, Count(Lag_MktVal) As n
				From CRSP5
				Group By YearMonth
				Order By YearMonth;
			Quit;

			Proc Means Data=CRSP5 noprint;
				Var Ret;
				By YearMonth;
				Weight Lag_MktVal;
				Output Out=Mean_Return Mean=Mean_Ret;
			Run;

			Proc Means Data=CRSP5 noprint;
				Var Retx;
				By YearMonth;
				Weight Lag_MktVal;
				Output Out=Mean_Returnx Mean=Mean_Retx;
			Run;

			Proc Means Data=CRSP5 mean NoPrint;
				Var AT_Growth BEME Debt_Growth EI_BE GP_AT Hire_Rate XRD_ME INVT_Growth INV_AT
					Debt_AT IK OC_AT ROA TFP IVOL ME;
				By YearMonth;
				Output Out=Mean_CHARACTER
					mean(AT_Growth)=Mean_AT_G
					mean(BEME)=Mean_BEME
					mean(Debt_Growth) = Mean_Debt_Growth
					mean(EI_BE) = Mean_EI_BE
					mean(GP_AT)=Mean_GP_AT
					mean(Hire_Rate)=Mean_Hire_Rate
					mean(XRD_ME)=Mean_XRD_ME
					mean(INVT_Growth)=Mean_INVT_G
					mean(INV_AT)=Mean_INV_AT
					mean(DEBT_AT)=Mean_DEBT_AT
					mean(IK)=Mean_IK
					Mean(OC_AT) = Mean_OC_AT
					mean(ROA)=Mean_ROA
					mean(TFP)=Mean_TFP
					mean(IVOL)=Mean_IVOL
					mean(ME)=Mean_ME;	
			Run;

			Proc SQL;	
				Create Table Industry%eval(&i) As
				Select a.YearMonth, a.Mean_Ret As Ret, b.Mean_Retx As Retx
				From Mean_Return as a, Mean_Returnx as b
				Where a.YearMonth=b.YearMonth And a.YearMonth>=196701;
			Quit;

			Proc SQL;
				Create Table Industry%eval(&i) As
				Select a.*, b.Total_LagMktVal As Lag_MktVal, b.n
				From Industry%eval(&i) as a, MktVal as b
				Where a.YearMonth=b.YearMonth And a.YearMonth>=196701;
			Quit;

			Proc SQL;
				Create Table Industry%eval(&i) As
				Select a.*, b.*
				From Industry%eval(&i) as a, Mean_CHARACTER as b
				Where a.YearMonth=b.YearMonth And a.YearMonth>=196701;
			Quit;

			Data Industry%eval(&i);
				Set Industry%eval(&i);
					Drop _TYPE_ _FREQ_;
			Run;

			* Since industry 29 is blank, then fill the spreadsheet with missing values;
			%If %Eval(&i)=29 %Then %Do;
				Proc SQL;
					Create Table Filler As
					Select YearMonth, -99 As Ret, 	-99 as Retx, 		-99 as Lag_MktVal, 		-99 As n,
							-99 as Mean_AT_G, 		-99 as Mean_BEME, 	-99 as Mean_Debt_Growth,
							-99 as Mean_EI_BE, 		-99 as Mean_GP_AT, 	-99 as Mean_Hire_Rate, 	-99 as Mean_XRD_ME,
							-99 as Mean_INVT_G, 	-99 as Mean_INV_AT, -99 as Mean_DEBT_AT, 	-99 as Mean_IK,
							-99 as Mean_OC_AT, 		-99 as Mean_ROA, 	-99 as Mean_TFP, 		-99 as Mean_IVOL,
							-99 as Mean_ME
					From Industry5;
				Quit;
				
				Proc SQL;
					Create Table Industry%eval(&i) As
					Select a.YearMonth, b.*
					From Filler as a Left Join Industry%eval(&i) as b
					On a.YearMonth=b.YearMonth;
				Quit;
			%End;

			Data Industry%eval(&i);
				Set Industry%eval(&i);
					If Missing(Ret) Then Ret=-99;
					If Missing(Retx) Then Retx=-99;
					If Missing(Lag_MktVal) Then Lag_MktVal=-99;
					If Missing(n) Then n=-99;
					If Missing(Mean_AT_G) Then Mean_AT_G=-99;
					If Missing(Mean_BEME) Then Mean_BEME=-99;
					If Missing(Mean_Debt_Growth) Then Mean_Debt_Growth=-99;
					If Missing(Mean_EI_BE) Then Mean_EI_BE=-99;		
					If Missing(Mean_GP_AT) Then Mean_GP_AT=-99;
					If Missing(Mean_Hire_Rate) Then Mean_Hire_Rate=-99;
					If Missing(Mean_XRD_ME) Then Mean_XRD_ME=-99;
					If Missing(Mean_INVT_G) Then Mean_INVT_G=-99;
					If Missing(Mean_INV_AT) Then Mean_INV_AT=-99;
					If Missing(Mean_DEBT_AT) Then Mean_DEBT_AT=-99;
					If Missing(Mean_IK) Then Mean_IK=-99;
					If Missing(Mean_OC_AT) Then Mean_OC_AT=-99;
					If Missing(Mean_ROA) Then Mean_ROA=-99;
					If Missing(Mean_TFP) Then Mean_TFP=-99;
					If Missing(Mean_ME) Then Mean_ME=-99;
					If Missing(Mean_IVOL) Then Mean_IVOL=-99;
			Run;

			Proc Export Data=Industry%eval(&i) DBMS=CSV Outfile="<LOCAL-DIRECTORY>\Data\Industry-files\Industry&i..csv"
				Replace;
				PutNames=No;
			Run;
	%End;

	%Do i=48 %To 54;
			Data CRSP5;
				Set CRSP4;
					If Industry%eval(&i)=1;
						Proc Sort;
							By Permno YearMonth;
			Run;
			
			Data CRSP5;
				Set CRSP5;
					Lag_Permno=Lag(Permno);
					Lag_Date=Lag(Date);
					Lag_MktVal=Lag(MKTVAL);
					Format Lag_Date YYMMDDN8.;
			Run;
			
			Data CRSP5 (Drop=Lag_Permno Lag_Date);
				Set CRSP5;
					If intck('day',Lag_Date,Date)<=40 & Permno=Lag_Permno Then Output;
					PRoc Sort;
						By YearMonth;
			Run;

			Proc SQL;
				Create Table MktVal As
				Select YearMonth, Sum(Lag_MktVal) As Total_LagMktVal, Count(Lag_MktVal) As n
				From CRSP5
				Group By YearMonth
				Order By YearMonth;
			Quit;

			Proc Means Data=CRSP5 noprint;
				Var Ret;
				By YearMonth;
				Weight Lag_MktVal;
				Output Out=Mean_Return Mean=Mean_Ret;
			Run;

			Proc Means Data=CRSP5 noprint;
				Var Retx;
				By YearMonth;
				Weight Lag_MktVal;
				Output Out=Mean_Returnx Mean=Mean_Retx;
			Run;

			Proc Means Data=CRSP5 mean NoPrint;
				Var AT_Growth BEME Debt_Growth EI_BE GP_AT Hire_Rate XRD_ME INVT_Growth INV_AT
					Debt_AT IK OC_AT ROA TFP IVOL ME;
				By YearMonth;
				Output Out=Mean_CHARACTER
					mean(AT_Growth)=Mean_AT_G
					mean(BEME)=Mean_BEME
					mean(Debt_Growth) = Mean_Debt_Growth
					mean(EI_BE) = Mean_EI_BE
					mean(GP_AT)=Mean_GP_AT
					mean(Hire_Rate)=Mean_Hire_Rate
					mean(XRD_ME)=Mean_XRD_ME
					mean(INVT_Growth)=Mean_INVT_G
					mean(INV_AT)=Mean_INV_AT
					mean(DEBT_AT)=Mean_DEBT_AT
					mean(IK)=Mean_IK
					Mean(OC_AT) = Mean_OC_AT
					mean(ROA)=Mean_ROA
					mean(TFP)=Mean_TFP
					mean(IVOL)=Mean_IVOL
					mean(ME)=Mean_ME;	
			Run;

			Proc SQL;	
				Create Table Industry%eval(&i) As
				Select a.YearMonth, a.Mean_Ret As Ret, b.Mean_Retx As Retx
				From Mean_Return as a, Mean_Returnx as b
				Where a.YearMonth=b.YearMonth And a.YearMonth>=196701;
			Quit;

			Proc SQL;
				Create Table Industry%eval(&i) As
				Select a.*, b.Total_LagMktVal As Lag_MktVal, b.n
				From Industry%eval(&i) as a, MktVal as b
				Where a.YearMonth=b.YearMonth And a.YearMonth>=196701;
			Quit;

			Proc SQL;
				Create Table Industry%eval(&i) As
				Select a.*, b.*
				From Industry%eval(&i) as a, Mean_CHARACTER as b
				Where a.YearMonth=b.YearMonth And a.YearMonth>=196701;
			Quit;

			Data Industry%eval(&i);
				Set Industry%eval(&i);
					Drop _TYPE_ _FREQ_;
			Run;

			* Since industry 54 is blank, then fill the spreadsheet with missing values;
			%If %Eval(&i)=54 %Then %Do;
				Proc SQL;
					Create Table Filler As
					Select YearMonth, -99 As Ret, 	-99 as Retx, 		-99 as Lag_MktVal, 		-99 As n,
							-99 as Mean_AT_G, 		-99 as Mean_BEME, 	-99 as Mean_Debt_Growth,
							-99 as Mean_EI_BE, 		-99 as Mean_GP_AT, 	-99 as Mean_Hire_Rate, 	-99 as Mean_XRD_ME,
							-99 as Mean_INVT_G, 	-99 as Mean_INV_AT, -99 as Mean_DEBT_AT, 	-99 as Mean_IK,
							-99 as Mean_OC_AT, 		-99 as Mean_ROA, 	-99 as Mean_TFP, 		-99 as Mean_IVOL,
							-99 as Mean_ME
					From Industry5;
				Quit;
				
				Proc SQL;
					Create Table Industry%eval(&i) As
					Select a.YearMonth, b.*
					From Filler as a Left Join Industry%eval(&i) as b
					On a.YearMonth=b.YearMonth;
				Quit;
			%End;

			Data Industry%eval(&i);
				Set Industry%eval(&i);
					If Missing(Ret) Then Ret=-99;
					If Missing(Retx) Then Retx=-99;
					If Missing(Lag_MktVal) Then Lag_MktVal=-99;
					If Missing(n) Then n=-99;
					If Missing(Mean_AT_G) Then Mean_AT_G=-99;
					If Missing(Mean_BEME) Then Mean_BEME=-99;
					If Missing(Mean_Debt_Growth) Then Mean_Debt_Growth=-99;
					If Missing(Mean_EI_BE) Then Mean_EI_BE=-99;		
					If Missing(Mean_GP_AT) Then Mean_GP_AT=-99;
					If Missing(Mean_Hire_Rate) Then Mean_Hire_Rate=-99;
					If Missing(Mean_XRD_ME) Then Mean_XRD_ME=-99;
					If Missing(Mean_INVT_G) Then Mean_INVT_G=-99;
					If Missing(Mean_INV_AT) Then Mean_INV_AT=-99;
					If Missing(Mean_DEBT_AT) Then Mean_DEBT_AT=-99;
					If Missing(Mean_IK) Then Mean_IK=-99;
					If Missing(Mean_OC_AT) Then Mean_OC_AT=-99;
					If Missing(Mean_ROA) Then Mean_ROA=-99;
					If Missing(Mean_TFP) Then Mean_TFP=-99;
					If Missing(Mean_IVOL) Then Mean_IVOL=-99;
					If Missing(Mean_ME) Then Mean_ME=-99;
					
			Run;

			Proc Export Data=Industry%eval(&i) DBMS=CSV Outfile="<LOCAL-DIRECTORY>\Data\Industry-files\Industry&i..csv"
				Replace;
				PutNames=No;
			Run;
	%End;
%Mend IndLoop;
%IndLoop;


