function [grid, Transition] = Rouwenhorst(mu, rho, sigma, N)
    % Calculate the moments of the AR(1) Process
    meanVal = mu / (1-rho);
    
    % Calc differential nu
    nu = sqrt((N-1)/(1-rho^2))*sigma;
    
    % Construct grid
    grid = linspace(meanVal - nu, meanVal + nu,N);
    
    p = (1+rho)/2;
    q = (1+rho)/2;
    T = [p 1-p; 1-q q];
    for i = 3:N
        z = zeros(i-1,1);
        T_New = p*[T z ; z' 0]  + (1-p) *[z T; 0 z'] + (1-q)*[z' 0 ; T z] + q*[0 z' ; z T];
        T_New(2:end-1,:) = T_New(2:end-1,:) /2;
        T = T_New;
    end
    Transition = T;
end

