% s = 1     Baseline [Firm-level + Long sample]
% s = 2     Baseline [Firm-level + Short-sample]
% s = 3     Baseline [Ind-level + Short-sample];
% s = 4     Baseline [Ind-level + Long-sample];
% s = 5     No utilization
% s = 6     Very low lambda
% s = 7     Low lambda
% s = 8     High lambda
% s = 9     Very high lambda
% s = 10    Fixed utilization + Very low phi
% s = 11    Fixed utilization + Low phi
% s = 12    Fixed utilization + High phi
% s = 13    Fixed utilization + Very high phi
% s = 14    Very low phi
% s = 15    Low phi
% s = 16    High phi
% s = 17    Very high phi
%
for s=[1:17]
    load CALIBRATION.mat
    tic

    f_div = 0.0275;
    N_Z   = 11; 
    N_A   = 5;  
    N_K   = 201;
    FIX_UTIL = CALIBRATION(1,s);

    %%%% Production function parameters %%%%;
    rts       = 0.95;       % Returns to scale parameter;
    alpha_k   = (1/3)*rts;  % Share of capital in production function;
    alpha_l   = (2/3)*rts;  % Share of labor in the production function;
    
    %%%% Shock parameters %%%%;
    rho_a     = 0.922;      % Persistence of agg. shock;
    sigma_a   = 0.014;      % Vol. of agg. shock;
    rho_z     = 0.6;        % Persistence of idio. shock
    sigma_z     = 0.30;
    mean_a      = 0;
    %%%% Depreciation and utilization parameters %%%%;
    delta_k   = 0.08;                       % Depreciation rate of physical capital;
    eta       = CALIBRATION(2,s);           % Adjustment cost for capital;         

    %%%% Wage parameters %%%%;
    omega     = 0.20;                       % Wage sensitivity to agg. shock;

    %%%% SDF and preference parameters %%%%;
    beta      = 0.988;               
    zeta      = CALIBRATION(3,s);                 % Sensitivty of utilization;
    % Setting the fixed and variable component of the market price of risk;
    gamma_0   = 3.375;
    gamma_1   = -8.80;

    [zss, delta_u] = deterministicSS_FC(mean_a/(1-rho_a),beta,delta_k, omega, eta, gamma_0, gamma_1,alpha_k,alpha_l,zeta,sigma_a);
    mean_z = zss*(1-rho_z);
    
    % Saving the parameters of the calibration;
    CALIBATION_PARAMETERS = NaN(17,1);
    CALIBATION_PARAMETERS(1) = rho_a;
    CALIBATION_PARAMETERS(2) = sigma_a;
    CALIBATION_PARAMETERS(3) = rho_z;
    CALIBATION_PARAMETERS(4) = sigma_z;
    CALIBATION_PARAMETERS(5) = zss;
    CALIBATION_PARAMETERS(6) = beta;
    CALIBATION_PARAMETERS(7) = gamma_0;
    CALIBATION_PARAMETERS(8) = gamma_1;
    CALIBATION_PARAMETERS(9) = alpha_k/rts;
    CALIBATION_PARAMETERS(10) = alpha_l/rts;
    CALIBATION_PARAMETERS(11) = rts;
    CALIBATION_PARAMETERS(12) = delta_k;
    CALIBATION_PARAMETERS(13) = delta_u;
    CALIBATION_PARAMETERS(14) = zeta;
    CALIBATION_PARAMETERS(15) = omega;
    CALIBATION_PARAMETERS(16) = eta;
    CALIBATION_PARAMETERS(17) = f_div;
    if s==1
        save('Output/CALIBATION_PARAMETERS','CALIBATION_PARAMETERS');
    end
    
    % Setting the grid for K;
    min_k = 0.40;
    max_k = 60;   
    k_grid = exp(linspace(log(min_k), log(max_k), N_K))';
    k_grid_min = exp(linspace(log(0.1),log(min_k),150))';
    k_grid_max = exp(linspace(log(max_k),log(max_k+50),150))';
    k_grid=[k_grid_min(1:end-1);k_grid;k_grid_max(2:end)];    
    N_K = size(k_grid,1);

    % Setting the grid for z
    z_grid = linspace(zss-1.965*sqrt(sigma_z^2/(1-rho_z^2)),zss+1.965*sqrt(sigma_z^2/(1-rho_z^2)),N_Z)';
    N_Z = size(z_grid,1);
    P_z =  Tauchen(zss*(1-rho_z), rho_z, sigma_z, z_grid);

    % Setting the grid for A;
    [a_grid, P_a] = Rouwenhorst(mean_a, rho_a, sigma_a, N_A);
    a_grid = a_grid';

    % Combining the grids for the productivity shocksl
    az_grid = [kron(ones(N_Z,1),a_grid), kron(z_grid,ones(N_A,1))];
    P_az = kron(P_z,P_a);
    N_AZ = N_A*N_Z;

    SDF_Prime = zeros(N_AZ,N_AZ);
    for i_az = 1:N_AZ
        curr_A = exp(az_grid(i_az,1));
        curr_Z = exp(az_grid(i_az,2));
        curr_Gamma = gamma_0+gamma_1*log(curr_A);
        for i_az_prime = 1:N_AZ
            next_A = exp(az_grid(i_az_prime,1));
            shock_A = log(next_A)-((1-rho_a)*mean_a+rho_a*log(curr_A));
            SDF_Prime(i_az,i_az_prime)  = exp(-exp(curr_Gamma)*shock_A-(1/2)*exp(curr_Gamma)^2*sigma_a^2);
        end
        SDF_Prime(i_az,:) = beta*SDF_Prime(i_az,:);
    end

    DiscountFactor  = P_az.*SDF_Prime;
    Rf              = log(1/beta)*ones(N_AZ,1);
    wage            = zeros(N_K*N_AZ, 1);
    labor           = zeros(N_K*N_AZ, 1);
    utilization     = zeros(N_K*N_AZ, 1);
    output          = zeros(N_K*N_AZ, 1);
    depreciation    = zeros(N_K,N_AZ, 1);
    investment      = zeros(N_K*N_AZ, N_K);
    adjcost         = zeros(N_K*N_AZ, N_K);
    fcDisinv        = zeros(N_K*N_AZ, N_K);
    div             = zeros(N_K*N_AZ, N_K);

    for i_az = 1:N_AZ
        for i_k = 1:N_K
            i_state = i_k + (i_az-1)*N_K;
            curr_A = exp(az_grid(i_az,1));
            curr_Z = exp(az_grid(i_az,2));
            curr_K = k_grid(i_k);

            curr_Wage  = exp(omega*log(curr_A));
            if( ~FIX_UTIL)
                curr_U = ...
                    ((curr_A*curr_Z*alpha_k/delta_u)*curr_K^(alpha_k-1)*curr_Wage^(alpha_l/(alpha_l-1))*...
                    (curr_A*curr_Z*curr_K^(alpha_k)*alpha_l)^(alpha_l/(1-alpha_l)))^((1-alpha_l)/(zeta*(1-alpha_l)+1-alpha_k-alpha_l));
                curr_L = (curr_Wage/(curr_A*curr_Z*(curr_U*curr_K)^(alpha_k)*alpha_l))^(1/(alpha_l-1));
                curr_Delta = delta_k+delta_u*(curr_U^(1+zeta)-1)/(1+zeta);
            else
                curr_Delta = delta_k;
                curr_U = 1;
                curr_L = (curr_Wage/(curr_A*curr_Z*(curr_K)^(alpha_k)*alpha_l))^(1/(alpha_l-1));
            end
            curr_Output = curr_A*curr_Z*(curr_U*curr_K)^alpha_k*curr_L^alpha_l;
            depreciation(i_state)   = curr_Delta;
            wage(i_state)           = curr_Wage;
            labor(i_state)          = curr_L;
            utilization(i_state)    = curr_U;
            output(i_state)         = curr_Output;
            for i_k_prime = 1:N_K
                curr_Inv = (k_grid(i_k_prime) - (1-curr_Delta)*k_grid(i_k));
                investment(i_state, i_k_prime) = curr_Inv;
                if (i_k_prime<i_k)
                    fcDisinv(i_state, i_k_prime) = f_div*curr_K;
                    adjcost(i_state, i_k_prime) = 0.5*eta*(curr_Inv/curr_K-curr_Delta)^2*curr_K;
                end
                if (i_k_prime>i_k)
                    adjcost(i_state, i_k_prime) = 0.5*eta*(curr_Inv/curr_K-curr_Delta)^2*curr_K;
                end
                div(i_state,i_k_prime) = curr_Output - curr_Inv - adjcost(i_state, i_k_prime) - curr_L*curr_Wage-fcDisinv(i_state, i_k_prime);
            end
        end
    end
    V       = output;
    TV      = zeros(N_K*N_AZ,1);
    tol     = 1e-10;
    diffV   = 1;
    iter    = 1;
    while (diffV>tol)
        EMV             = kron(DiscountFactor*(reshape(V,N_K,N_AZ)'),ones(N_K,1));
        [TV, POLICY_K]  = max(div + EMV,[],2);
        diffV           = max(norm(V-TV,Inf));
        V               = TV;
        fprintf('iter %.0f: diffV is %.07f\n',iter,diffV);
        iter = iter + 1;
    end
    
    assert(sum(V<=0)==0);
    
    if (s>=3 && s<=4) 
        simulate_cs_IND
    else
        simulate_cs
    end
end

