function UnivPortfolioRets = univariate_ports(UnivPortSort,Rf,Rm)
    % Input the following objects:
    %   (1)     Portfolio sort structure (inc. returns + characteristics);
    %   (2)     The complete path of the risk-free rate;
    %   (3)     A proxy for the market portfolio;
    portfolioRetsVW         = UnivPortSort.PortRetsVW;

    assert(size(portfolioRetsVW,1) == size(Rf,1) &&...
        size(portfolioRetsVW,1) == size(Rm,1));

    NaN_Index                   = sum(isnan(portfolioRetsVW),2)==0;    
    portfolioRetsVW     = portfolioRetsVW(NaN_Index,:);
    Rf      = Rf(NaN_Index);
    Rm      = Rm(NaN_Index);    
    
    portfolioRetsVW(:,end+1)=portfolioRetsVW(:,1)-portfolioRetsVW(:,end);
    
    P = size(portfolioRetsVW,2);
    
    CAPM_VW     = NaN(P,3);
      
    for p=1:P
        if p<size(portfolioRetsVW,2)
            oVW=ols(portfolioRetsVW(:,p)-Rf,[ones(size(Rf,1),1) Rm],0);
        else
            oVW=ols(portfolioRetsVW(:,p),[ones(size(Rf,1),1) Rm],0);      
        end 
        CAPM_VW(p,1) = mean(portfolioRetsVW(:,p));  % Report the portfolio's mean return;
        CAPM_VW(p,3) = oVW.beta(2);                 % Report the portfolio's CAPM beta
    end
    UnivPortfolioRets.CAPM_VW   = CAPM_VW;
end

