function UnivPortfolioSort = univariate_sort(SortVar_CS,Ret_CS,...
    breakpoint_percentiles,W_CS)

    TS = size(Ret_CS,1);
    CS = size(Ret_CS,2);
    N_BP            = max(size(breakpoint_percentiles));
    N_PORTFOLIO     = N_BP + 1;
    portfolioSorts  = NaN(TS,CS);
    
    for t = 2:TS
        measures_curr = SortVar_CS(t-1,~isnan(SortVar_CS(t-1,:)));
        breakpoints = zeros(N_BP,1);
        for i = 1:N_BP
            breakpoints(i) = prctile(measures_curr,breakpoint_percentiles(i));
        end
        index_L = (SortVar_CS(t-1,:)<=breakpoints(1));
        portfolioSorts(t,index_L) = 1;
        
        index_H = (SortVar_CS(t-1,:)>breakpoints(N_BP));
        portfolioSorts(t,index_H) = N_BP+1;
        
        for p=2:N_BP
            index_P = SortVar_CS(t-1,:)<=breakpoints(p) & SortVar_CS(t-1,:)>breakpoints(p-1);
            portfolioSorts(t,index_P) = p;
        end
    end
    
    portfolioAssignments = portfolioSorts;
    portfolioRetsVW     = NaN(TS,N_PORTFOLIO);    
    portfolioRetsEW     = NaN(TS,N_PORTFOLIO);
    
    for t = 2:TS
        for i = 1:N_PORTFOLIO
            nFirms      = nansum(portfolioAssignments(t,:) == i);
            rets_in_portfolio   = NaN(nFirms,1);
            lmvs_in_portfolio   = NaN(nFirms,1);
            PortfolioIndex = (portfolioAssignments(t,:) == i);
            rets_in_portfolio(:,1) = Ret_CS(t,PortfolioIndex);
            lmvs_in_portfolio(:,1) = W_CS(t-1,PortfolioIndex);
            if isempty(rets_in_portfolio')
                portfolioRetsVW(t,i)    = NaN;
            else
                portfolioRetsVW(t,i)    = rets_in_portfolio'*(lmvs_in_portfolio./sum(lmvs_in_portfolio));
            end 
        end
    end 
    UnivPortfolioSort.PortRetsVW    = portfolioRetsVW;
    UnivPortfolioSort.PortAssign    = portfolioAssignments;
end










